

def show(filename):
    import importlib.resources as pkg_resources
    from IPython.display import display, Image
    package = "theory"
    filename += '.png'
    try:
        with pkg_resources.path(package, filename) as file_path:
            img = Image(filename=str(file_path))
            display(img)
    except Exception as e:
        print(f'Неправильное имя файла: {e}')
    return filename


def show_pdf(filename):
    import importlib.resources as pkg_resources
    from IPython.display import display, IFrame
    package = "numpyp.theory"
    filename += '.pdf'
    try:
        with pkg_resources.path(package, filename) as file_path:
            # Создаем IFrame для отображения PDF
            pdf_iframe = IFrame(src=str(file_path), width=1000, height=800)
            display(pdf_iframe)
    except Exception as e:
        print(f'Неправильное имя файла: {e}')
    return filename

def vec():
    show('1')
    show('2')
    show('3')
    show('4')


def func1():
    print('1 z_1_1() переполнение')
    print('1 z_1_2() Потеря точности')
    print('1 z_1_3() Ошибки округления')
    print('1 z_1_4() Накопление ошибок')
    print('1 z_1_5() Потеря значимости')
    print('1 z_1_6() Суммирование по Кахану')
    print('1 z_1_7() Суммирование по Кахану vs обычное vs numpy')

def func2():
    print('2 z_2_1() Метод простых итераций  ')
    print('2 z_2_2() Метод хорд ')
    print('2 z_2_3() Метод вегстейна')
    print('2 z_2_4() Метод ньютона ')

def func3():
    print('3 z_3() Метод дихотомии')

def func4():
    print('4 z_4_1() Простые итерации ')
    print('4 z_4_2() Метод зейделя')
    print('4 z_4_3() Метод ньютона ')
    print('4 z_4_4() Модифицированный метод ньютона')

def func5():
    print('5 z_5_1() определения ')
    print('5 z_5_2() простая интерполяция')
    print('5 z_5_3() линейная интерполяция')
    print('5 z_5_4() интерполяционные полиномы ')
    print('5 z_5_5() интерполяционный многочлен Лагранжа ')
    print('5 z_5_6() Интерполяция Кубический сплайн ')
    print('5 z_5_7() Квадратичная интерполяция')
    print('5 z_5_8() Интерполяция сплайнами ')
    print('5 z_5_9() Метод наименьших квадратов')







# # # numpyp/__init__.py
# #
# # import os
# # from dotenv import load_dotenv
# # from .telegram_handler import _TelegramHandler
# #
# # # ... (код _telegram_instance и _auto_initialize_if_needed остается БЕЗ ИЗМЕНЕНИЙ) ...
# # _telegram_instance: _TelegramHandler | None = None
# #
# #
# # def _auto_initialize_if_needed():
# #     # ЭТОТ КОД НЕ МЕНЯЕТСЯ
# #     global _telegram_instance
# #     if _telegram_instance: return
# #     print("🔧 Первая попытка вызова Telegram-функции. Автоматическая настройка...")
# #     library_path = os.path.dirname(__file__)
# #     dotenv_path = os.path.join(library_path, '.env')
# #     if not os.path.exists(dotenv_path): raise FileNotFoundError(
# #         f"Не найден .env файл внутри библиотеки! Поместите .env по этому пути: {library_path}")
# #     load_dotenv(dotenv_path=dotenv_path)
# #     token = os.getenv("TELEGRAM_TOKEN")
# #     chat_id = os.getenv("TELEGRAM_CHAT_ID")
# #     if not token or not chat_id: raise ValueError("TELEGRAM_TOKEN или TELEGRAM_CHAT_ID не найдены в вашем .env файле.")
# #     _telegram_instance = _TelegramHandler(token=token, chat_id=chat_id)
# #     print("✅ Telegram успешно авто-инициализирован из .env файла библиотеки.")
# #
# #
# # # --- КЛЮЧЕВОЕ ИЗМЕНЕНИЕ ---
# #
# # async def call(text: str):
# #     """
# #     Асинхронно отправляет сообщение в настроенный Telegram-чат.
# #     """
# #     _auto_initialize_if_needed()
# #     # Теперь здесь нужен await
# #     await _telegram_instance.send_message(text)
# #
# #
# # async def ans() -> str | None:
# #     """
# #     Асинхронно проверяет наличие ответа на последнее сообщение.
# #     """
# #     _auto_initialize_if_needed()
# #     # И здесь тоже нужен await
# #     reply = await _telegram_instance.check_for_reply()
# #
# #     if reply:
# #         print(f"Ответ: {reply}")
# #     else:
# #         print("Ответа пока нет.")
# #
# #     return reply  # Возвращаем ответ для возможности его использования в коде
# # numpyp/__init__.py
#
# # numpyp/__init__.py
#
# import os
# from dotenv import load_dotenv
# from .telegram_handler import _TelegramHandler
#
# # Глобальная переменная модуля, которая будет хранить единственный экземпляр обработчика.
# _telegram_instance: _TelegramHandler | None = None
#
#
# def _auto_initialize_if_needed():
#     """
#     Автоматически настраивает модуль при первом вызове,
#     загружая данные из .env файла, который лежит внутри самой библиотеки.
#     """
#     global _telegram_instance
#     if _telegram_instance:
#         return
#
#     print("🔧 Первая попытка вызова. Автоматическая настройка...")
#
#     # Находим путь к папке, где установлена сама библиотека
#     library_path = os.path.dirname(__file__)
#     dotenv_path = os.path.join(library_path, '.env')
#
#     if not os.path.exists(dotenv_path):
#         raise FileNotFoundError(f"Не найден .env файл внутри библиотеки! Поместите .env по этому пути: {library_path}")
#
#     load_dotenv(dotenv_path=dotenv_path)
#
#     # Сначала просто получаем все значения как строки
#     token = os.getenv("TELEGRAM_TOKEN")
#     chat_id = os.getenv("TELEGRAM_CHAT_ID")
#     redis_host = os.getenv("REDIS_HOST")
#     redis_port_str = os.getenv("REDIS_PORT")
#
#     # Проверяем, что ни одно из значений не пустое
#     if not all([token, chat_id, redis_host, redis_port_str]):
#         raise ValueError(
#             "Одна или несколько настроек (TELEGRAM_TOKEN, TELEGRAM_CHAT_ID, REDIS_HOST, REDIS_PORT) не найдены или пусты в .env файле."
#         )
#
#     # Теперь, когда мы уверены, что redis_port_str не пустой, превращаем его в число
#     try:
#         redis_port = int(redis_port_str)
#     except ValueError:
#         raise TypeError(f"Значение REDIS_PORT ('{redis_port_str}') в .env файле не является корректным числом.")
#
#     # Создаем экземпляр обработчика с проверенными данными
#     _telegram_instance = _TelegramHandler(token=token, chat_id=chat_id, redis_host=redis_host, redis_port=redis_port)
#     print("✅ Модуль успешно авто-инициализирован.")
#
#
# async def call(text: str, task_id: str):
#     """
#     Асинхронно отправляет сообщение и регистрирует задачу с уникальным ID.
#     :param text: Текст сообщения.
#     :param task_id: Уникальный строковый идентификатор этой задачи.
#     """
#     _auto_initialize_if_needed()
#     await _telegram_instance.send_message(text, task_id)
#
#
# async def ans(task_id: str) -> str | None:
#     """
#     Асинхронно проверяет наличие ответа для задачи с уникальным ID.
#     :param task_id: Уникальный идентификатор задачи, ответ на которую мы ждем.
#     """
#     _auto_initialize_if_needed()
#     reply = await _telegram_instance.check_for_reply(task_id)
#
#     if reply:
#         print(f"Получен ответ для задачи '{task_id}': {reply}")
#     else:
#         print(f"Ответа для задачи '{task_id}' пока нет.")
#
#     return reply

import os
from dotenv import load_dotenv
from .telegram_handler import _TelegramHandler
import pyperclip
_telegram_instance: _TelegramHandler | None = None


def _auto_initialize_if_needed():
    """
    Автоматически настраивает модуль при первом вызове,
    загружая данные из .env файла, который лежит внутри самой библиотеки.
    """
    global _telegram_instance
    if _telegram_instance:
        return

    print("🔧 Первая попытка вызова. Автоматическая настройка...")

    library_path = os.path.dirname(__file__)
    dotenv_path = os.path.join(library_path, '.env')

    if not os.path.exists(dotenv_path):
        raise FileNotFoundError(f"Не найден .env файл внутри библиотеки! Поместите .env по этому пути: {library_path}")

    load_dotenv(dotenv_path=dotenv_path)

    token = os.getenv("TELEGRAM_TOKEN")
    chat_id = os.getenv("TELEGRAM_CHAT_ID")
    redis_host = os.getenv("REDIS_HOST")
    redis_port_str = os.getenv("REDIS_PORT")

    if not all([token, chat_id, redis_host, redis_port_str]):
        raise ValueError(
            "Одна или несколько настроек (TELEGRAM_TOKEN, TELEGRAM_CHAT_ID, REDIS_HOST, REDIS_PORT) не найдены или пусты в .env файле.")

    try:
        redis_port = int(redis_port_str)
    except ValueError:
        raise TypeError(f"Значение REDIS_PORT ('{redis_port_str}') в .env файле не является корректным числом.")

    _telegram_instance = _TelegramHandler(token=token, chat_id=chat_id, redis_host=redis_host, redis_port=redis_port)
    print("✅ Модуль успешно авто-инициализирован.")


async def call(text: str, task_id: str):
    """
    Асинхронно отправляет сообщение и регистрирует задачу с уникальным ID.
    :param text: Текст сообщения.
    :param task_id: Уникальный строковый идентификатор этой задачи.
    """
    _auto_initialize_if_needed()
    await _telegram_instance.send_message(text, task_id)


async def ans(task_id: str) -> dict | None:
    """
    Асинхронно проверяет наличие ответа для задачи с уникальным ID.
    Возвращает словарь с информацией об ответе или None.
    """
    _auto_initialize_if_needed()
    reply_data = await _telegram_instance.check_for_reply(task_id)

    if reply_data:
        answer = f"""{reply_data['text']}"""
        pyperclip.copy(answer)
        pyperclip.paste()
    else:
        print(f"Ответа для задачи '{task_id}' пока нет.")

    return reply_data


# #
# # import telegram
# # import redis
# #
# #
# # class _TelegramHandler:
# #     """
# #     Внутренний класс-обработчик.
# #     Управляет взаимодействием с Telegram API и хранилищем состояний Redis.
# #     """
# #
# #     def __init__(self, token: str, chat_id: str, redis_host: str, redis_port: int):
# #         self.bot = telegram.Bot(token=token)
# #         self.chat_id = chat_id
# #         # Подключаемся к Redis. decode_responses=True чтобы получать строки, а не байты.
# #         try:
# #             self.redis_client = redis.Redis(host=redis_host, port=redis_port, decode_responses=True)
# #             # Проверяем подключение, чтобы сразу выявить проблему, если Redis недоступен
# #             self.redis_client.ping()
# #             print("✅ Подключение к Redis успешно.")
# #         except redis.exceptions.ConnectionError as e:
# #             print(f"❌ Не удалось подключиться к Redis: {e}")
# #             raise ConnectionError("Проверьте, что Redis запущен и доступен по указанному хосту и порту.") from e
# #
# #     async def send_message(self, text: str, task_id: str):
# #         """Асинхронно отправляет сообщение и регистрирует задачу в Redis."""
# #         try:
# #             message = await self.bot.send_message(chat_id=self.chat_id, text=text)
# #             # Сохраняем ID сообщения в Redis с ключом задачи.
# #             # Ставим срок жизни ключа - 24 часа (86400 секунд), чтобы не засорять базу старыми задачами.
# #             self.redis_client.set(f"task:{task_id}", message.message_id, ex=86400)
# #             print(f"✅ Сообщение для задачи '{task_id}' отправлено. ID: {message.message_id}")
# #         except Exception as e:
# #             print(f"❌ Ошибка отправки сообщения для задачи '{task_id}': {e}")
# #
# #     async def check_for_reply(self, task_id: str) -> str | None:
# #         """Асинхронно проверяет ответ для конкретной задачи."""
# #         # Получаем ID сообщения из Redis по ключу задачи
# #         message_id_to_check_str = self.redis_client.get(f"task:{task_id}")
# #
# #         if not message_id_to_check_str:
# #             # Задача не найдена - возможно, уже выполнена или время жизни истекло.
# #             return None
# #
# #         message_id_to_check = int(message_id_to_check_str)
# #
# #         try:
# #             updates = await self.bot.get_updates(timeout=1)
# #             for update in reversed(updates):
# #                 msg = update.message
# #                 # Ищем ответ на наше конкретное сообщение
# #                 if (msg and msg.reply_to_message and
# #                         msg.reply_to_message.message_id == message_id_to_check):
# #                     print(f"✅ Найден ответ для задачи '{task_id}'.")
# #                     # Задача выполнена, удаляем ее из Redis, чтобы не проверять снова.
# #                     self.redis_client.delete(f"task:{task_id}")
# #                     return msg.text
# #             return None  # Ответа пока нет
# #         except Exception as e:
# #             print(f"❌ Ошибка получения ответа для задачи '{task_id}': {e}")
# #             return None
#
# # numpyp/telegram_handler.py
#
# import telegram
# import redis
#
#
# class _TelegramHandler:
#     def __init__(self, token: str, chat_id: str, redis_host: str, redis_port: int):
#         self.bot = telegram.Bot(token=token)
#         self.chat_id = chat_id
#         try:
#             self.redis_client = redis.Redis(host=redis_host, port=redis_port, decode_responses=True)
#             self.redis_client.ping()
#             print("✅ Подключение к Redis успешно.")
#         except redis.exceptions.ConnectionError as e:
#             print(f"❌ Не удалось подключиться к Redis: {e}")
#             raise ConnectionError("Проверьте, что Redis запущен и доступен по указанному хосту и порту.") from e
#
#     async def send_message(self, text: str, task_id: str):
#         """Асинхронно отправляет форматированное сообщение и регистрирует задачу."""
#         # --- ИЗМЕНЕНИЕ 1: ФОРМАТИРУЕМ ТЕКСТ ---
#         formatted_text = f"--{task_id}--\n\n{text}"
#
#         try:
#             message = await self.bot.send_message(chat_id=self.chat_id, text=formatted_text)
#             # Ключ для основного сообщения задачи. Срок жизни 24 часа.
#             task_key = f"task:{task_id}"
#             self.redis_client.set(task_key, message.message_id, ex=86400)
#
#             # Также установим срок жизни для набора обработанных ответов, чтобы он не висел вечно
#             processed_replies_key = f"task:{task_id}:processed_replies"
#             self.redis_client.expire(processed_replies_key, 86400)
#
#             print(f"✅ Сообщение для задачи '{task_id}' отправлено. ID: {message.message_id}")
#         except Exception as e:
#             print(f"❌ Ошибка отправки сообщения для задачи '{task_id}': {e}")
#
#     async def check_for_reply(self, task_id: str) -> str | None:
#         """Асинхронно проверяет НОВЫЙ, еще не обработанный ответ для задачи."""
#         task_key = f"task:{task_id}"
#         processed_replies_key = f"task:{task_id}:processed_replies"
#
#         message_id_to_check_str = self.redis_client.get(task_key)
#         if not message_id_to_check_str:
#             return None  # Задачи не существует
#
#         message_id_to_check = int(message_id_to_check_str)
#
#         try:
#             updates = await self.bot.get_updates(timeout=1)
#             # --- ИЗМЕНЕНИЕ 2: ЛОГИКА МНОЖЕСТВЕННЫХ ОТВЕТОВ ---
#             processed_reply_ids = self.redis_client.smembers(processed_replies_key)
#
#             for update in reversed(updates):
#                 msg = update.message
#                 if (msg and msg.reply_to_message and
#                         msg.reply_to_message.message_id == message_id_to_check):
#
#                     # Проверяем, видели ли мы уже этот ответ
#                     if str(msg.message_id) not in processed_reply_ids:
#                         print(f"✅ Найден НОВЫЙ ответ для задачи '{task_id}'. ID ответа: {msg.message_id}")
#                         # Добавляем ID этого ответа в набор "обработанных"
#                         self.redis_client.sadd(processed_replies_key, msg.message_id)
#                         return msg.text  # Возвращаем текст нового ответа
#
#             return None  # Новых неотвеченных сообщений нет
#         except Exception as e:
#             print(f"❌ Ошибка получения ответа для задачи '{task_id}': {e}")
#             return None

import telegram
import redis


class _TelegramHandler:
    """
    Внутренний класс-обработчик.
    Управляет взаимодействием с Telegram API и хранилищем состояний Redis.
    """

    def __init__(self, token: str, chat_id: str, redis_host: str, redis_port: int):
        self.bot = telegram.Bot(token=token)
        self.chat_id = chat_id
        try:
            self.redis_client = redis.Redis(host=redis_host, port=redis_port, decode_responses=True)
            self.redis_client.ping()
            print("✅ Подключение к Redis успешно.")
        except redis.exceptions.ConnectionError as e:
            print(f"❌ Не удалось подключиться к Redis: {e}")
            raise ConnectionError("Проверьте, что Redis запущен и доступен по указанному хосту и порту.") from e

    async def send_message(self, text: str, task_id: str):
        """Асинхронно отправляет форматированное сообщение и регистрирует задачу."""
        formatted_text = f"--{task_id}--\n\n{text}"

        try:
            message = await self.bot.send_message(chat_id=self.chat_id, text=formatted_text)
            task_key = f"task:{task_id}"
            processed_replies_key = f"task:{task_id}:processed_replies"

            # Сохраняем ID основного сообщения. Срок жизни 24 часа.
            self.redis_client.set(task_key, message.message_id, ex=86400)
            # Устанавливаем срок жизни для набора ответов, чтобы он тоже удалился.
            self.redis_client.expire(processed_replies_key, 86400)

            print(f"✅ Сообщение для задачи '{task_id}' отправлено. ID: {message.message_id}")
        except Exception as e:
            print(f"❌ Ошибка отправки сообщения для задачи '{task_id}': {e}")

    async def check_for_reply(self, task_id: str) -> dict | None:
        """Асинхронно проверяет НОВЫЙ, еще не обработанный ответ для задачи."""
        task_key = f"task:{task_id}"
        processed_replies_key = f"task:{task_id}:processed_replies"

        message_id_to_check_str = self.redis_client.get(task_key)
        if not message_id_to_check_str:
            return None  # Задачи не существует или ее время жизни истекло

        message_id_to_check = int(message_id_to_check_str)

        try:
            updates = await self.bot.get_updates(timeout=1)
            processed_reply_ids = self.redis_client.smembers(processed_replies_key)

            for update in reversed(updates):
                msg = update.message
                if (msg and msg.reply_to_message and
                        msg.reply_to_message.message_id == message_id_to_check):

                    if str(msg.message_id) not in processed_reply_ids:
                        print(f"✅ Найден НОВЫЙ ответ для задачи '{task_id}'. ID ответа: {msg.message_id}")
                        self.redis_client.sadd(processed_replies_key, msg.message_id)

                        # Собираем словарь с информацией об авторе ответа и текстом
                        return {
                            "username": msg.from_user.username or "N/A",
                            "first_name": msg.from_user.first_name,
                            "text": msg.text,
                            "user_id": msg.from_user.id
                        }

            return None  # Новых необработанных ответов нет
        except Exception as e:
            print(f"❌ Ошибка получения ответа для задачи '{task_id}': {e}")
            return None